/*
* Copyright (C) 2004 Ladislav Vitasek
* info@wordrider.net
* http://www.wordrider.net
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
package net.wordrider.area.actions;

import javax.swing.*;
import javax.swing.text.Document;
import javax.swing.text.JTextComponent;
import java.awt.event.ActionEvent;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;

/**
 * @author Vity
 */
public final class SelectAllAction extends TextAreaAction {
    private final static SelectAllAction instance = new SelectAllAction();
    private final static String CODE = "SelectAllAction";

    private SelectAllAction() {
        super(CODE, KeyStroke.getKeyStroke(KeyEvent.VK_A, InputEvent.CTRL_MASK), null);
    }

    public static SelectAllAction getInstance() {
        return instance;
    }

    /**
     * The operation to perform when this action is triggered.
     */
    public final void actionPerformed(final ActionEvent e) {
        final JTextComponent target = getTextComponent(e);
        if (target != null) {
            final Document doc = target.getDocument();
            target.setCaretPosition(0);
            target.moveCaretPosition(doc.getLength());
        }
    }

    public boolean isEnabled() {
        final JTextComponent comp = getFocusedComponent();
        return comp != null && comp.isEnabled()
                && comp.getText().length() > 0;
    }

}
